from django.apps import AppConfig


class OttuConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "ottu.contrib.django"
    label = "ottu"
    verbose_name = "Ottu"
